@@ -3,7 +3,6 @@ use std::slice;
 use std::i32;
 use std::collections::{BTreeMap, HashMap};
 use std::marker::PhantomData;
-use std::cell::{Cell, UnsafeCell};
 use std::hash::Hash;
 
 use webcore::ffi;
@@ -15,6 +14,7 @@ use webcore::type_name::type_name;
 use webcore::symbol::Symbol;
 use webcore::unsafe_typed_array::UnsafeTypedArray;
 use webcore::once::Once;
+use webcore::global_arena;
 
 use webcore::value::{
     Null,
@@ -49,70 +49,9 @@ impl Default for Tag {
     }
 }
 
-#[doc(hidden)]
-#[derive(Debug)]
-pub struct PreallocatedArena {
-    saved: UnsafeCell< Vec< Value > >,
-    buffer: UnsafeCell< Vec< u8 > >,
-    index: Cell< usize >
-}
-
-impl PreallocatedArena {
-    #[doc(hidden)]
-    #[inline]
-    pub fn new( memory_required: usize ) -> Self {
-        let mut buffer = Vec::new();
-        buffer.reserve( memory_required );
-        unsafe {
-            buffer.set_len( memory_required );
-        }
-
-        PreallocatedArena {
-            saved: UnsafeCell::new( Vec::new() ),
-            buffer: UnsafeCell::new( buffer ),
-            index: Cell::new( 0 )
-        }
-    }
-
-    #[doc(hidden)]
-    #[inline]
-    pub fn reserve< T >( &self, count: usize ) -> &mut [T] {
-        let bytes = mem::size_of::< T >() * count;
-        let slice = unsafe {
-            let buffer = &mut *self.buffer.get();
-            debug_assert!( self.index.get() + bytes <= buffer.len() );
-
-            slice::from_raw_parts_mut(
-                buffer.as_mut_ptr().offset( self.index.get() as isize ) as *mut T,
-                count
-            )
-        };
-
-        self.index.set( self.index.get() + bytes );
-        slice
-    }
-
-    #[doc(hidden)]
-    #[inline]
-    pub fn save( &self, value: Value ) -> &Value {
-        unsafe {
-            let saved = &mut *self.saved.get();
-            saved.push( value );
-            &*(saved.last().unwrap() as *const Value)
-        }
-    }
-
-    #[doc(hidden)]
-    #[inline]
-    pub fn assert_no_free_space_left( &self ) {
-        debug_assert!( self.index.get() == unsafe { &*self.buffer.get() }.len() );
-    }
-}
-
 #[doc(hidden)]
 pub trait JsSerializeOwned: Sized {
-    fn into_js_owned< 'a >( value: &'a mut Option< Self >, arena: &'a PreallocatedArena ) -> SerializedValue< 'a >;
-    fn memory_required_owned( &self ) -> usize;
+    fn into_js_owned< 'a >( value: &'a mut Option< Self > ) -> SerializedValue< 'a >;
 }
 
 /// A trait for types which can be serialized through the `js!` macro.
@@ -121,10 +60,7 @@ pub trait JsSerializeOwned: Sized {
 /// to be used inside generic code for specifying trait bounds.
 pub trait JsSerialize {
     #[doc(hidden)]
-    fn _into_js< 'a >( &'a self, arena: &'a PreallocatedArena ) -> SerializedValue< 'a >;
-
-    #[doc(hidden)]
-    fn _memory_required( &self ) -> usize;
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a >;
 }
 
 // This is a generic structure for serializing every JavaScript value.
@@ -479,15 +415,9 @@ impl< 'a > SerializedValue< 'a > {
 impl JsSerialize for () {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedUndefined.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        0
-    }
 }
 
 __js_serializable_boilerplate!( () );
@@ -495,15 +425,9 @@ __js_serializable_boilerplate!( () );
 impl JsSerialize for Undefined {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedUndefined.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        0
-    }
 }
 
 __js_serializable_boilerplate!( Undefined );
@@ -511,15 +435,9 @@ __js_serializable_boilerplate!( Undefined );
 impl JsSerialize for Null {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedNull.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        0
-    }
 }
 
 __js_serializable_boilerplate!( Null );
@@ -527,17 +445,11 @@ __js_serializable_boilerplate!( Null );
 impl JsSerialize for Symbol {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedSymbol {
             id: self.0
         }.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        0
-    }
 }
 
 __js_serializable_boilerplate!( Symbol );
@@ -545,17 +457,11 @@ __js_serializable_boilerplate!( Symbol );
 impl JsSerialize for Reference {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedReference {
             refid: self.as_raw()
         }.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        0
-    }
 }
 
 __js_serializable_boilerplate!( Reference );
@@ -563,19 +469,13 @@ __js_serializable_boilerplate!( Reference );
 impl JsSerialize for bool {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         if *self {
             SerializedUntaggedTrue {}.into()
         } else {
             SerializedUntaggedFalse {}.into()
         }
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        0
-    }
 }
 
 __js_serializable_boilerplate!( bool );
@@ -583,18 +483,12 @@ __js_serializable_boilerplate!( bool );
 impl JsSerialize for str {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedString {
             pointer: self.as_ptr() as u32,
             length: self.len() as u32
         }.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        0
-    }
 }
 
 __js_serializable_boilerplate!( impl< 'a > for &'a str );
@@ -602,14 +496,8 @@ __js_serializable_boilerplate!( impl< 'a > for &'a str );
 impl JsSerialize for String {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, arena: &'a PreallocatedArena ) -> SerializedValue< 'a > {
-        self.as_str()._into_js( arena )
-    }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        self.as_str()._memory_required()
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
+        self.as_str()._into_js()
     }
 }
 
@@ -618,17 +506,11 @@ __js_serializable_boilerplate!( String );
 impl JsSerialize for i8 {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedI32 {
             value: *self as i32
         }.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        (*self as i32)._memory_required()
-    }
 }
 
 __js_serializable_boilerplate!( i8 );
@@ -636,17 +518,11 @@ __js_serializable_boilerplate!( i8 );
 impl JsSerialize for i16 {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedI32 {
             value: *self as i32
         }.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        (*self as i32)._memory_required()
-    }
 }
 
 __js_serializable_boilerplate!( i16 );
@@ -654,17 +530,11 @@ __js_serializable_boilerplate!( i16 );
 impl JsSerialize for i32 {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedI32 {
             value: *self
         }.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        0
-    }
 }
 
 __js_serializable_boilerplate!( i32 );
@@ -672,17 +542,11 @@ __js_serializable_boilerplate!( i32 );
 impl JsSerialize for u8 {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedI32 {
             value: *self as i32
         }.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        (*self as i32)._memory_required()
-    }
 }
 
 __js_serializable_boilerplate!( u8 );
@@ -690,17 +554,11 @@ __js_serializable_boilerplate!( u8 );
 impl JsSerialize for u16 {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedI32 {
             value: *self as i32
         }.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        (*self as i32)._memory_required()
-    }
 }
 
 __js_serializable_boilerplate!( u16 );
@@ -708,17 +566,11 @@ __js_serializable_boilerplate!( u16 );
 impl JsSerialize for u32 {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedF64 {
             value: *self as f64
         }.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        (*self as f64)._memory_required()
-    }
 }
 
 __js_serializable_boilerplate!( u32 );
@@ -726,17 +578,11 @@ __js_serializable_boilerplate!( u32 );
 impl JsSerialize for f32 {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedF64 {
             value: *self as f64
         }.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        (*self as f64)._memory_required()
-    }
 }
 
 __js_serializable_boilerplate!( f32 );
@@ -744,17 +590,11 @@ __js_serializable_boilerplate!( f32 );
 impl JsSerialize for f64 {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         SerializedUntaggedF64 {
             value: *self
         }.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        0
-    }
 }
 
 __js_serializable_boilerplate!( f64 );
@@ -762,19 +602,13 @@ __js_serializable_boilerplate!( f64 );
 impl JsSerialize for Number {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, arena: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         use webcore::number::{Storage, get_storage};
         match *get_storage( self ) {
-            Storage::I32( ref value ) => value._into_js( arena ),
-            Storage::F64( ref value ) => value._into_js( arena )
+            Storage::I32( ref value ) => value._into_js(),
+            Storage::F64( ref value ) => value._into_js()
         }
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        0
-    }
 }
 
 __js_serializable_boilerplate!( Number );
@@ -782,23 +616,13 @@ __js_serializable_boilerplate!( Number );
 impl< T: JsSerialize > JsSerialize for Option< T > {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, arena: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         if let Some( value ) = self.as_ref() {
-            value._into_js( arena )
+            value._into_js()
         } else {
             SerializedUntaggedNull.into()
         }
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        if let Some( value ) = self.as_ref() {
-            value._memory_required()
-        } else {
-            0
-        }
-    }
 }
 
 __js_serializable_boilerplate!( impl< T > for Option< T > where T: JsSerialize );
@@ -806,24 +630,19 @@ __js_serializable_boilerplate!( impl< T > for Option< T > where T: JsSerialize )
 impl< T: JsSerialize > JsSerialize for [T] {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, arena: &'a PreallocatedArena ) -> SerializedValue< 'a > {
-        let output = arena.reserve( self.len() );
-        for (value, output_value) in self.iter().zip( output.iter_mut() ) {
-            *output_value = value._into_js( arena );
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
+        let mut output = global_arena::reserve( self.len() );
+        for value in self {
+            unsafe {
+                output.append( value._into_js() );
+            }
         }
 
         SerializedUntaggedArray {
-            pointer: output.as_ptr() as u32,
+            pointer: output.offset() as u32,
             length: output.len() as u32
         }.into()
     }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        mem::size_of::< SerializedValue >() * self.len() +
-        self.iter().fold( 0, |sum, value| sum + value._memory_required() )
-    }
 }
 
 __js_serializable_boilerplate!( impl< 'a, T > for &'a [T] where T: JsSerialize );
@@ -831,51 +650,35 @@ __js_serializable_boilerplate!( impl< 'a, T > for &'a [T] where T: JsSerialize )
 impl< T: JsSerialize > JsSerialize for Vec< T > {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, arena: &'a PreallocatedArena ) -> SerializedValue< 'a > {
-        self.as_slice()._into_js( arena )
-    }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        self.as_slice()._memory_required()
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
+        self.as_slice()._into_js()
     }
 }
 
 __js_serializable_boilerplate!( impl< T > for Vec< T > where T: JsSerialize );
 
-fn object_into_js< 'a, K: AsRef< str >, V: 'a + JsSerialize, I: Iterator< Item = (K, &'a V) > + ExactSizeIterator >( iter: I, arena: &'a PreallocatedArena ) -> SerializedValue< 'a > {
-    let keys = arena.reserve( iter.len() );
-    let values = arena.reserve( iter.len() );
-    for (((key, value), output_key), output_value) in iter.zip( keys.iter_mut() ).zip( values.iter_mut() ) {
-        *output_key = key.as_ref()._into_js( arena ).as_string().clone();
-        *output_value = value._into_js( arena );
+fn object_into_js< 'a, K: AsRef< str >, V: 'a + JsSerialize, I: Iterator< Item = (K, &'a V) > + ExactSizeIterator >( iter: I ) -> SerializedValue< 'a > {
+    let mut keys = global_arena::reserve( iter.len() );
+    let mut values = global_arena::reserve( iter.len() );
+    for (key, value) in iter {
+        unsafe {
+            keys.append( key.as_ref()._into_js().as_string().clone() );
+            values.append( value._into_js() );
+        }
     }
 
     SerializedUntaggedObject {
-        key_pointer: keys.as_ptr() as u32,
-        value_pointer: values.as_ptr() as u32,
+        key_pointer: keys.offset() as u32,
+        value_pointer: values.offset() as u32,
         length: keys.len() as u32
     }.into()
 }
 
-fn object_memory_required< K: AsRef< str >, V: JsSerialize, I: Iterator< Item = (K, V) > + ExactSizeIterator >( iter: I ) -> usize {
-    mem::size_of::< SerializedValue >() * iter.len() +
-    mem::size_of::< SerializedUntaggedString >() * iter.len() +
-    iter.fold( 0, |sum, (key, value)| sum + key.as_ref()._memory_required() + value._memory_required() )
-}
-
 impl< K: AsRef< str >, V: JsSerialize > JsSerialize for BTreeMap< K, V > {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, arena: &'a PreallocatedArena ) -> SerializedValue< 'a > {
-        object_into_js( self.iter(), arena )
-    }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        object_memory_required( self.iter() )
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
+        object_into_js( self.iter() )
     }
 }
 
@@ -884,14 +687,8 @@ __js_serializable_boilerplate!( impl< K, V > for BTreeMap< K, V > where K: AsRef
 impl< K: AsRef< str > + Eq + Hash, V: JsSerialize > JsSerialize for HashMap< K, V > {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'a >( &'a self, arena: &'a PreallocatedArena ) -> SerializedValue< 'a > {
-        object_into_js( self.iter(), arena )
-    }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        object_memory_required( self.iter() )
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
+        object_into_js( self.iter() )
     }
 }
 
@@ -899,28 +696,15 @@ __js_serializable_boilerplate!( impl< K, V > for HashMap< K, V > where K: AsRef<
 
 impl JsSerialize for Value {
     #[doc(hidden)]
-    fn _into_js< 'a >( &'a self, arena: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
         match *self {
             Value::Undefined => SerializedUntaggedUndefined.into(),
             Value::Null => SerializedUntaggedNull.into(),
-            Value::Bool( ref value ) => value._into_js( arena ),
-            Value::Number( ref value ) => value._into_js( arena ),
-            Value::Symbol( ref value ) => value._into_js( arena ),
-            Value::String( ref value ) => value._into_js( arena ),
-            Value::Reference( ref value ) => value._into_js( arena )
-        }
-    }
-
-    #[doc(hidden)]
-    fn _memory_required( &self ) -> usize {
-        match *self {
-            Value::Undefined => Undefined._memory_required(),
-            Value::Null => Null._memory_required(),
-            Value::Bool( value ) => value._memory_required(),
-            Value::Number( value ) => value._memory_required(),
-            Value::Symbol( ref value ) => value._memory_required(),
-            Value::String( ref value ) => value._memory_required(),
-            Value::Reference( ref value ) => value._memory_required()
+            Value::Bool( ref value ) => value._into_js(),
+            Value::Number( ref value ) => value._into_js(),
+            Value::Symbol( ref value ) => value._into_js(),
+            Value::String( ref value ) => value._into_js(),
+            Value::Reference( ref value ) => value._into_js()
         }
     }
 }
@@ -932,19 +716,13 @@ macro_rules! impl_for_unsafe_typed_array {
         impl< 'r > JsSerialize for UnsafeTypedArray< 'r, $ty > {
             #[doc(hidden)]
             #[inline]
-            fn _into_js< 'a >( &'a self, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+            fn _into_js< 'a >( &'a self ) -> SerializedValue< 'a > {
                 SerializedUntaggedUnsafeTypedArray {
                     pointer: self.0.as_ptr() as u32 / mem::size_of::< $ty >() as u32,
                     length: self.0.len() as u32,
                     kind: $kind
                 }.into()
             }
-
-            #[doc(hidden)]
-            #[inline]
-            fn _memory_required( &self ) -> usize {
-                0
-            }
         }
 
         __js_serializable_boilerplate!( impl< 'a > for UnsafeTypedArray< 'a, $ty > );
@@ -968,13 +746,8 @@ pub struct NonFunctionTag;
 
 impl< T: JsSerialize > JsSerializeOwned for Newtype< (NonFunctionTag, ()), T > {
     #[inline]
-    fn into_js_owned< 'x >( value: &'x mut Option< Self >, arena: &'x PreallocatedArena ) -> SerializedValue< 'x > {
-        JsSerialize::_into_js( value.as_ref().unwrap().as_ref(), arena )
-    }
-
-    #[inline]
-    fn memory_required_owned( &self ) -> usize {
-        JsSerialize::_memory_required( &**self )
+    fn into_js_owned< 'x >( value: &'x mut Option< Self > ) -> SerializedValue< 'x > {
+        JsSerialize::_into_js( value.as_ref().unwrap().as_ref() )
     }
 }
 
@@ -1026,10 +799,9 @@ macro_rules! impl_for_fn {
                 $crate::private::noop( &mut nth_argument );
 
                 let result = callback.call_mut( ($($kind,)*) );
-                let mut arena = PreallocatedArena::new( result.memory_required_owned() );
 
                 let mut result = Some( result );
-                let result = JsSerializeOwned::into_js_owned( &mut result, &mut arena );
+                let result = JsSerializeOwned::into_js_owned( &mut result );
                 let result = &result as *const _;
 
                 // This is kinda hacky but I'm not sure how else to do it at the moment.
@@ -1090,10 +862,9 @@ macro_rules! impl_for_fn {
                 $crate::private::noop( &mut nth_argument );
 
                 let result = callback.call_once( ($($kind,)*) );
-                let mut arena = PreallocatedArena::new( result.memory_required_owned() );
 
                 let mut result = Some( result );
-                let result = JsSerializeOwned::into_js_owned( &mut result, &mut arena );
+                let result = JsSerializeOwned::into_js_owned( &mut result );
                 let result = &result as *const _;
 
                 // This is kinda hacky but I'm not sure how else to do it at the moment.
@@ -1113,63 +884,52 @@ macro_rules! impl_for_fn {
             where F: CallOnce< ($($kind,)*) > + 'static, F::Output: JsSerializeOwned
         {
             #[inline]
-            fn into_js_owned< 'a >( value: &'a mut Option< Self >, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+            fn into_js_owned< 'a >( value: &'a mut Option< Self > ) -> SerializedValue< 'a > {
                 let callback: *mut F = Box::into_raw( Box::new( value.take().unwrap().unwrap_newtype().0 ) );
-
+                let adapter_pointer = <Self as FuncallAdapter< F > >::funcall_adapter;
+                let deallocator_pointer = <Self as FuncallAdapter< F > >::deallocator;
                 SerializedUntaggedFunctionOnce {
-                    adapter_pointer: <Self as FuncallAdapter< F > >::funcall_adapter as u32,
+                    adapter_pointer: adapter_pointer as u32,
                     pointer: callback as u32,
-                    deallocator_pointer: <Self as FuncallAdapter< F > >::deallocator as u32
+                    deallocator_pointer: deallocator_pointer as u32
                 }.into()
             }
-
-            #[inline]
-            fn memory_required_owned( &self ) -> usize {
-                0
-            }
         }
 
         impl< $($kind: TryFrom< Value >,)* F > JsSerializeOwned for Newtype< (FunctionTag, ($($kind,)*)), F >
             where F: CallMut< ($($kind,)*) > + 'static, F::Output: JsSerializeOwned
         {
             #[inline]
-            fn into_js_owned< 'a >( value: &'a mut Option< Self >, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+            fn into_js_owned< 'a >( value: &'a mut Option< Self > ) -> SerializedValue< 'a > {
                 let callback: *mut F = Box::into_raw( Box::new( value.take().unwrap().unwrap_newtype() ) );
-
+                let adapter_pointer = <Self as FuncallAdapter< F > >::funcall_adapter;
+                let deallocator_pointer = <Self as FuncallAdapter< F > >::deallocator;
                 SerializedUntaggedFunction {
-                    adapter_pointer: <Self as FuncallAdapter< F > >::funcall_adapter as u32,
+                    adapter_pointer: adapter_pointer as u32,
                     pointer: callback as u32,
-                    deallocator_pointer: <Self as FuncallAdapter< F > >::deallocator as u32
+                    deallocator_pointer: deallocator_pointer as u32
                 }.into()
             }
-
-            #[inline]
-            fn memory_required_owned( &self ) -> usize {
-                0
-            }
         }
 
         impl< $($kind: TryFrom< Value >,)* F > JsSerializeOwned for Newtype< (FunctionTag, ($($kind,)*)), Option< F > >
             where F: CallMut< ($($kind,)*) > + 'static, F::Output: JsSerializeOwned
         {
             #[inline]
-            fn into_js_owned< 'a >( value: &'a mut Option< Self >, _: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+            fn into_js_owned< 'a >( value: &'a mut Option< Self > ) -> SerializedValue< 'a > {
                 if let Some( value ) = value.take().unwrap().unwrap_newtype() {
                     let callback: *mut F = Box::into_raw( Box::new( value ) );
+                    let adapter_pointer = <Newtype< (FunctionTag, ($($kind,)*)), F > as FuncallAdapter< F > >::funcall_adapter;
+                    let deallocator_pointer = <Newtype< (FunctionTag, ($($kind,)*)), F > as FuncallAdapter< F > >::deallocator;
                     SerializedUntaggedFunction {
-                        adapter_pointer: <Newtype< (FunctionTag, ($($kind,)*)), F > as FuncallAdapter< F > >::funcall_adapter as u32,
+                        adapter_pointer: adapter_pointer as u32,
                         pointer: callback as u32,
-                        deallocator_pointer: <Newtype< (FunctionTag, ($($kind,)*)), F > as FuncallAdapter< F > >::deallocator as u32
+                        deallocator_pointer: deallocator_pointer as u32
                     }.into()
                 } else {
                     SerializedUntaggedNull.into()
                 }
             }
-
-            #[inline]
-            fn memory_required_owned( &self ) -> usize {
-                0
-            }
         }
 
         next! { $next }
@@ -1181,14 +941,8 @@ loop_through_identifiers!( impl_for_fn );
 impl< 'a, T: ?Sized + JsSerialize > JsSerialize for &'a T {
     #[doc(hidden)]
     #[inline]
-    fn _into_js< 'x >( &'x self, arena: &'x PreallocatedArena ) -> SerializedValue< 'x > {
-        T::_into_js( *self, arena )
-    }
-
-    #[doc(hidden)]
-    #[inline]
-    fn _memory_required( &self ) -> usize {
-        T::_memory_required( *self )
+    fn _into_js< 'x >( &'x self ) -> SerializedValue< 'x > {
+        T::_into_js( *self )
     }
 }
 
@@ -1765,6 +1519,19 @@ mod test_serialization {
     test_unsafe_typed_array!( test_unsafe_typed_array_f64, f64, Float64Array );
 }
 
+// TODO: Move this back inside the test module.
+//
+// This had to be temporarily moved here due to a bug in Rust
+// where the following error is generated if it's defined under
+// the module:
+//
+//    error: cannot determine resolution for the attribute macro `reference`
+//
+#[cfg(test)]
+#[derive(Clone, Debug, PartialEq, Eq, ReferenceType)]
+#[reference(instance_of = "Error")]
+pub struct TestError( Reference );
+
 #[cfg(test)]
 mod test_reserialization {
     use super::*;
@@ -1926,9 +1693,7 @@ mod test_reserialization {
         assert_eq!( non_empty, Value::String( "死神はりんごしか食べない!".to_string() ) );
     }
 
-    #[derive(Clone, Debug, PartialEq, Eq, ReferenceType)]
-    #[reference(instance_of = "Error")]
-    pub struct Error( Reference );
+    type Error = TestError;
 
     #[test]
     fn closure_returning_reference_object() {
